﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_check_primality
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Perform the tests to see if the number is prime.
        private void btnTest_Click(object sender, EventArgs e)
        {
            txtIsPrime.Clear();
            txtFactors.Clear();
            Cursor = Cursors.WaitCursor;
            Refresh();

            // Get the possible prime p.
            int p;
            if (!int.TryParse(txtNumber.Text, out p)) p = -1;
            if ((p < 3) || (p > 100000000))
            {
                MessageBox.Show("Number must be between 3 and 100,000,000");
                txtNumber.Focus();
                return;
            }

            // Perform the tests.
            int num_tests = (int)nudNumTests.Value;
            if (IsProbablyPrime(p, num_tests))
            {
                // It is probably prime.
                double prob = Math.Pow(2, -num_tests);
                txtIsPrime.Text = "Probably prime";
                txtFactors.Text = "Prob. composite: " + prob.ToString("R");
            }
            else
            {
                // It is not prime.
                // Display the factors.
                List<long> factors = FindFactors(p);
                List<string> strings = factors.ConvertAll<string>(x => x.ToString());
                txtFactors.Text = string.Join(" x ", strings.ToArray()); 
            }
            Cursor = Cursors.Default;
        }

        // Perform tests to see if a number is (probably) prime.
        private Random Rand = new Random();
        private bool IsProbablyPrime(int p, int num_tests)
        {
            checked
            {
                // Perform the tests.
                for (int i = 0; i < num_tests; i++)
                {
                    // Pick a number n in the range (1, p).
                    long n = Rand.Next(2, p);

                    // Calculate n ^ (p - 1).
                    long result = n;
                    for (int power = 1; power < p - 1; power++)
                    {
                        result = (result * n) % p;
                    }

                    // If the final result is not 1, p is not prime.
                    if (result != 1)
                    {
                        txtIsPrime.Text = (i + 1).ToString() +
                            " tests. " + n + "^(p-1) = " + result;
                        return false;
                    }
                }
            }

            // If we survived all the tests, p is probably prime.
            return true;
        }

        // Return the number's prime factors.
        private List<long> FindFactors(long num)
        {
            List<long> result = new List<long>();

            // Take out the 2s.
            while (num % 2 == 0)
            {
                result.Add(2);
                num /= 2;
            }

            // Take out other primes.
            long factor = 3;
            while (factor * factor <= num)
            {
                if (num % factor == 0)
                {
                    // This is a factor.
                    result.Add(factor);
                    num /= factor;
                }
                else
                {
                    // Go to the next odd number.
                    factor += 2;
                }
            }

            // If num is not 1, then whatever is left is prime.
            if (num > 1) result.Add(num);

            return result;
        }
    }
}
